#!/usr/bin/env python
#
# $Id: control.py,v 1.1.1.1 2003/02/11 06:07:42 resolve Exp $
#
# a selection of functions and their associated control strings, for doing
# various stuff with muse.
#
# Started 27/9/01 by Damien Elmes <resolve@repose.cx>
# This file is licensed under the GPL.
#

import player
import playlist
import string
import ui
import re
import os
import traceback

import muse

from debug import debug

callbacks = []

def queue_last():
    if not muse.plist.history:
        ui.notice("history is empty")
        return
    if muse.plist.selected == None:
        pos = 0
    else:
        pos = muse.plist.selected
        if pos == len(muse.plist)-1:
            pos+=1

    muse.plist.insert (pos, muse.plist.history[-1])
    del muse.plist.history[-1]
    muse.plist.selected = pos

def oldlist_cache (spec):
    elems = narrow_to_spec (spec)
    ret = []
    for f in elems:
        if isinstance (f, dict):
            ret.append (os.path.basename(f['path']) + "/")
        else:
            ret.append (os.path.basename(f))
    return ret

def as_lisp_list (ret):
    s = "("
    for l in ret:
        l = l.replace ('"', "\\\"")
        s = s + '"' + l + '" '
    if not ret: s = "()"
    else:
        s = s[:-1] + ")"
    return s

def narrow_to_spec_re (spec, cache=None):
    elems = string.split (spec, "/")
    dirs = elems[:-1]
    base = elems[-1]
    if cache==None:
        cache = muse.cache
    ret = []
    candidates = cache['dirs']
    for d in dirs:
        newcands = {}
        for (k,v) in candidates.items():
            if re.search (d, k, re.I):
                newcands[k]=v
        if not newcands:
            ui.notice ("unable to resolve " + d)
            return
        candidates = newcands
    # if there was no narrowing, use cache, not candidates
    if not dirs:
        l = [cache]
    else:
        l = candidates.values()

    for c in l:
        for (k, v) in c['dirs'].items():
            if re.search (base, k, re.I):
                ret.append (v)
            elif dirs:
                # only recurse if they want to
                ret.extend (narrow_to_spec (base, cache=v))
        for f in c['files']:
            if re.search (base, os.path.basename (f), re.I):
                ret.append (f)

def list_cache (spec=""):
    root = narrow_to_spec (spec)
    ret = []
    if isinstance (root, dict):
        for f in root['files']:
            ret.append(os.path.basename(f))
        for f in root['dirs'].keys():
            ret.append(os.path.basename(f))
    else:
        if isinstance (root, str):
            ret.append (os.path.basename(root))
    return ret

def narrow_to_spec (spec):
    print "narrowing to " + `spec`
    if spec.startswith("/"):
        spec = spec[1:]
    elems = string.split (spec, "/")
    tail = elems[-1]
    head = elems[:-1]
    if spec == "":
        head = []
        tail = []
    root = muse.cache
    for i in head:
        root = root['dirs'][i]
    if tail:
        if root['dirs'].has_key(tail):
            root = root['dirs'][tail]
        else:
            for f in root['files']:
                if os.path.basename(f) == tail:
                    return f
            debug("Invalid spec: %s" % spec)
            return None
    return root

def add_cache(spec=""):
    f = narrow_to_spec (spec)
    if isinstance (f, dict):
        muse.plist.add_cache (f)
    else:
        muse.plist.add_file (f)
    return "Added: " + `list_cache(spec)`

def applied (command):
    return lambda args: apply (command, args)

def applied0 (command):
    return lambda args=[]: apply (command, args)

def play (args):
    str = args[0]
    if len(args)>1:
        str += " " + args[1]
    if len(args)>2:
        str += ":" +  string.join (args[2:], ':')

    l = len (muse.plist)
    ret = cmd (str)
    muse.plist.selected = l
    muse.player.play()
    return "(playing) " + ret

def lisp_list_cache (spec=""):
    return as_lisp_list (list_cache (spec))

def playnext ():
    muse.player.play()
    s = muse.plist.playing
    if s:
        p = "Playing: " + s['song']
    else:
        p = "<nothing to play>"
    return p

control_list = {
    "p": play,
    "add": applied0(add_cache),
    "list": applied0(list_cache),
    "llist": applied0(lisp_list_cache),
    "clear": muse.plist.clear,
    "clear_hist": muse.plist.clear_history,
    "add_uri": applied(muse.plist.add_uri),
    "qlast": queue_last,

    # player modules
    "pause": muse.player.pause(),
    "stop": muse.player.stop(),
    "play": playnext,
    }

def cmd(str):
    """ Take a command - extra arguments are included in string. """

    if str.find("py: ")==0:
        # execute raw python code
        new = "_the_result=" + str[4:].strip()
        try:
            code = compile (new, "<string>", "single")
            eval(code, globals())
            return _the_result
        except:
            debug ("ERROR:")
            traceback.print_exc()
            return None

    else:
        foo = string.split(str.strip(), maxsplit=1)

        if foo:
            cmd = foo[0]
        else:
            cmd = ""

        if len(foo)>1:
            args = foo[1].split(":")
        else:
            args = []

        if control_list.has_key(cmd):
            report(cmd)
            try:
                if args:
                    return control_list[cmd](args)
                else:
                    return control_list[cmd]()
#                return apply(control_list[cmd], args)
            except TypeError, e:
                return e

        else:
            debug("unhandled command: %s (%s)" % (cmd, args))
            return "Invalid command! Try 'help' for a list."

def report(cmd):
    """ report a command to all registered hooks """
    for x in callbacks: x(cmd)

def add_cb(hook):
    """ add a cb for report() """
    callbacks.append(hook)
