#!/usr/bin/env python
#
# $Id: mp3id3.py,v 1.1.1.1 2003/02/11 06:07:41 resolve Exp $
#  
# this is based upon the work of mark pilgrim, from diveintopython.org
# - i've just added 'track' support and rewritten parts of it.
#
# Started 15/9/01 by Damien Elmes <resolve@repose.cx>
# This file is licensed under the GPL.
#

import os
import sys

def stripnulls(data):
    "strip whitespace and nulls"
    return data.replace("\00", " ").strip()

# oops, original author was only reading 29 chars, not 30, for comment
tagDataMap = {"title"   : (  3,  33, stripnulls),
              "artist"  : ( 33,  63, stripnulls),
              "album"   : ( 63,  93, stripnulls),
              "year"    : ( 93,  97, stripnulls),
              "comment" : ( 97, 127, None),
              "genre"   : (127, 128, ord)}

def parse(filename):
    "parse ID3v1.1 tags from MP3 file"
    data = {}
    
    try:
        fsock = open(filename, "rb", 0)
        try:
            fsock.seek(-128, 2)
            tagdata = fsock.read(128)
        finally:
            fsock.close()
        if tagdata[:3] == 'TAG':
            for tag, (start, end, parseFunc) in tagDataMap.items():
                if parseFunc:
                    data[tag] = parseFunc(tagdata[start:end])
                else:
                    data[tag] = tagdata[start:end]
        else:
            return None
    except IOError:
        return None

    if data['comment'][27] == ' ' and data['comment'][28] == '\00':
        data['track'] = ord(data['comment'][29])

    data['comment'] = stripnulls(data['comment'])

    return data

