#!/usr/bin/env python
#
# player.py - controls playing of music files. supports modules for playing
#             different types of files.
# $Id: player.py,v 1.1.1.1 2003/02/11 06:07:41 resolve Exp $
# started 5/8/01 by Damien Elmes <rapport@repose.cx>
#

import atexit
import os, sys
import time

import muse
import ui
import multiplex

import config

from debug import debug

config_defaults = {
    "persist_file" : os.path.expanduser("~/.muse/persist.dat"),
    "persist" : 1
    }

config.set_defaults("player", config_defaults)

PERSIST_FILE=config.get("player", "persist_file")

class Player:
    def __init__(self):
        self.plugins = []
        self.driver = None
        atexit.register(self.stop)

    def playing(self):
        if self.driver != None:
            return self.driver.is_playing()

    def add_plugin(self, plugin):
        self.plugins.append(plugin)

    def have_handler(self, file):
        """ iterate through our modules, and check to make sure a file is
        playable by at least one of them """

        for x in self.plugins:
            if x.validate_filename(file):
                return 1
        return 0

#     def filename_to_meta(self, file):
#         """ iterate through modules, returning the meta information from the
#         first available module """

#         for x in self.plugins:
#             if x.validate_filename(file):
#                 return x.filename_to_meta(file)
#         return None

    def play(self, p=None):
        if p==None: p=muse.plist.selected
        if p==None:
            if len(muse.plist) > 0:
                p=muse.plist.selected=0
            else:
                print "Nothing to play"
                return

        file = muse.plist[p]['path']
        if self.playing(): self.stop()
        self._play(file)
        muse.plist.playing = muse.plist[p]
        muse.plist.advance()

    def _play(self, file):
        """ Play a file"""
        # determine which player is capable of playing this file
        if not file:
            raise "player.play() was passed a playlist with no path."
        if not os.path.isfile(file):
            raise "player.play() was passed a file which does not exist: %s" % file

        debug("playing " + os.path.basename(file))



        for x in self.plugins:
            # fixme: add config option to validate via content
            if x.validate_filename(file):
                # set the current module for playing
                self.driver = x

                if config.get("player", "persist"):
                    # update the persist file so that if we're shut down, we
                    # can continue from this song. we store the size of the
                    # playlist for some very basic consistancy checking.

                    # doing this now is probably better than after we start
                    # playing, to prevent the poor laptop hard drive from
                    # skipping around.
                    path = os.path.dirname(sys.argv[0])
                    f = open(os.path.join(path,PERSIST_FILE), 'w')

                    f.write("%d\n" % (len(muse.plist),)) # playlist length
                    f.write("%s\n" % (file,)) # playing file

                    # i run ext3 on my car player, and don't bother shutting
                    # down, so it's nice to flush any buffers after we do this

                    f.flush()
                    os.fsync (f.fileno())
                    f.close()

                # start playing.. :)
                self.driver.play(file)

    def stop(self):
        if self.driver:
            self.driver.stop()
            muse.plist.playing = None

    def pause(self):
        if self.driver: self.driver.pause()

    def time_seek(self, secs):
        """ Seek to a time provided in seconds.

        This value will be normalised for your comfort, and seeks outside
        the current song length will result in the song being advanced forward
        or back. """

        if self.driver: self.driver.time_seek(secs)
        else: raise "time_seek with no current driver"

    def time_tell(self):
        """ Return the current playing time in seconds

        If there is nothing playing this time will be 0. """

        if self.driver: return self.driver.time_tell()
        else: return 0

    def time_total(self):
        """ Return total song time, -1 if unavailable. """

        if self.driver:
            tot = self.driver.time_total()
            if tot != 0:
                return tot

        return -1

    def finished(self):
        self.play ()

    def quit(self):
        self.stop()
        multiplex.running = 0

    def __getattr__(self, attr):

        if attr=="status":
            if self.driver:
                return self.driver.status
            raise "No current driver!"

        raise AttributeError, "no such attribute: %s" % attr

    def start(self, persist=0):
        """ called by the main function, just starts playing the first song in
        the playlist, if there is one

        note: if the file ~/.muse/persist.dat exists and is valid, then it will
        try to resume where we left off. if passed persist=0, then we move back
        to the start of the playlist """
        if len(muse.plist) == 0:
            ui.notice("playlist is empty")
            return

        if persist:
            # resume last pos
            try:
                f = open(PERSIST_FILE)

                plen = int(f.readline().strip())
                path = f.readline().strip()
            except:
                f = None

            if f and plen == len(muse.plist):
                # good, the playlists appear to be the same
                debug("attempting resume at file %s" % \
                      (os.path.basename(path),))

                # -1 is magic which makes it search entire list, including
                # first element
                index = muse.plist.jump_to_field(-1, 'path', path)
                if index >= 0:
                    muse.plist.selected=index
                else:
                    debug("resume failed")
        muse.plist.selected = 0
        self.play(muse.plist.selected)
