#!/usr/bin/env python
#
# $Id: ui_gtk2.py,v 1.1.1.1 2003/02/11 06:07:41 resolve Exp $
#
# Time-stamp: <2003-03-01 12:17:11 resolve>
#
# Copyright (C) Damien Elmes <resolve@repose.cx>, 2001.
# This file is licensed under the GPL. Please see COPYING for more details.
#

######### have muse inherit playlist and muse control etc so you can reuse


"""
GTK plugin for muse, with file selector dialog, isearch, etc.
"""

import pygtk
pygtk.require('2.0')
import gtk
GTK=gtk
try:
    from gtk import glade
    libglade = glade
except:
    import libglade
import gobject
import time
import os
import types

import muse
import ui
import multiplex
import control

from debug import debug

PLUGIN_TYPE="ui"

GLADE_FILE = "muse-gtk2.glade"

MOD_COLUMN_SONG=0
MOD_COLUMN_REF=1
MOD_COLUMN_BG=2

class IsearchWindow :
    def __init__(self):
        assert(os.path.isfile(GLADE_FILE))

        self.xml = libglade.XML(GLADE_FILE, root="isearch")

        # convenience
        self.isearch_window = self.xml.get_widget("isearch")
        self.isearch_clist = self.xml.get_widget("isearch_clist")
        self.isearch_entry = self.xml.get_widget("isearch_entry")

        self.isearch_len = 0
        self.isearch_previous = None

        self.isearch_window.show()

        self.isearch_model = gtk.ListStore(gobject.TYPE_STRING,
                                           gobject.TYPE_PYOBJECT,
                                           gobject.TYPE_STRING)

        self.isearch_treeview = gtk.TreeView(self.isearch_model)

        self.isearch_treeview.set_rules_hint (1)
        self.isearch_treeview.set_headers_visible(0)

        renderer = gtk.CellRendererText ()
        self.column = gtk.TreeViewColumn ('Song', renderer,
                                     text=MOD_COLUMN_SONG)
        self.isearch_treeview.append_column (self.column)

#        self.isearch_treeview.get_selection().connect("changed",
#                                self.on_isearch_treeview_select_row)
#        self.isearch_treeview.connect("button_press_event",
#                                self.on_isearch_treeview_button_press_event)

        dict = {}
        for key in self.__class__.__dict__.keys():
            if key.find("on_") != 0: continue

            val = getattr(self, key)
            if isinstance(val, types.MethodType):
                self.xml.signal_connect (key, val)

        parent = self.isearch_clist.get_parent()
        parent.remove(self.isearch_clist)
        parent.add(self.isearch_treeview)
        self.isearch_treeview.show()

    def on_close_activate(self, *args):
        self.isearch_window.hide()

    def on_isearch_entry_changed(self, *args):
        self.draw_isearch(muse.plist, self.isearch_entry.get_text())

    def draw_isearch(self, list, expr):
#        self.isearch_clist.freeze()

#        # if it's blank, clear the list of available options
        # (don't search for nothing)

        # split the expression and search on each word
        words = expr.split()

        if self.isearch_previous == None or \
           self.isearch_len > len(expr):
            # if there was no previous search, or they deleted a char, we
            # have to regen the list
            matchset = list
        else:
            matchset = self.isearch_previous

        self.isearch_len = len(expr)

        if expr == "":
            matchset = list
        else:
            self.isearch_model.clear()
            pass

        for word in words:
            newset = []
            for m in matchset:
                song = m['song'].lower()

                if (song.find(word.lower()) != -1):
                    # matched, add this row
                    iter = self.isearch_model.append()
                    self.isearch_model.set(iter, MOD_COLUMN_SONG,
                                      unicode(song, 'Latin-1'),
                                      MOD_COLUMN_REF, m, MOD_COLUMN_BG,
                                      "#FFFFFF")
                    if not newset:
                        path = self.isearch_model.get_path(iter)
                        self.isearch_treeview.set_cursor(path, self.column, 0)
                    newset.append(m)

            matchset = newset

        self.isearch_previous = matchset

        # now that we've narrowed matchset to the appropriate values, display

        for m in matchset:
            row = self.isearch_clist.append((m.song,))
            self.isearch_clist.set_row_data(row, m)

        self.selected_row = 0
#        self.isearch_clist.select_row(self.selected_row, 0)

        #self.isearch_clist.thaw()

    def on_next_song1_activate(self, *args):
        self.isearch_clist.select_row(self.selected_row+1, 0)

    def on_previous_song1_activate(self, *args):
        self.isearch_clist.select_row(self.selected_row-1, 0)

    def on_isearch_clist_select_row(self, object, row, *args):
        self.selected_row = row

    def on_play_current1_activate(self, *args):
        pass
#         iter = self.isearch_treeview.get_selection().get_selected()[1]
#         muse.plist.selected = muse.plist.index(self.isearch_model.get_value(iter, MOD_COLUMN_REF))
#         muse.player.play(muse.plist.selected)


    # self.on_close_activate(*args)

class PlaylistList:
    """ a GTK list which represents a playlist """

    def __init__(self, playlist):
        self.plist = playlist
        # create our tree and model
        self.model = gtk.ListStore(gobject.TYPE_STRING,
                                      gobject.TYPE_INT,
                                      gobject.TYPE_STRING)

        self.treeview = gtk.TreeView(self.model)

        self.treeview.set_rules_hint (1)
        self.treeview.set_headers_visible(0)

        renderer = gtk.CellRendererText ()
        self.column = gtk.TreeViewColumn ('Song', renderer,
                                     text=MOD_COLUMN_SONG,
                                     background=MOD_COLUMN_BG)
        self.treeview.append_column (self.column)

        self.treeview.get_selection().connect("changed",
                                self.on_treeview_select_row)
        self.treeview.connect("button_press_event",
                                self.on_treeview_button_press_event)

        self.plist.add_hook('list', self.draw_playlist)
        self.plist.add_hook('selected', self.update_current)

#        self.draw_playlist()

    def get_widget(self):
        return self.treeview

    def on_treeview_button_press_event(self, object, buttonevt, *args):
        if buttonevt.type == gtk.gdk._2BUTTON_PRESS:
            muse.player.play(self.plist.selected)

    def on_treeview_select_row(self, selection, *args):
        iter = selection.get_selected()[1]
        if iter:
            self.selected_iter = iter
            self.plist.__dict__['_selected'] = self.model.get_value(iter, MOD_COLUMN_REF)

    def draw_playlist(self):
        """ draw the entire playlist passed """
        self.model.clear()
        for l in range(len(self.plist)):
            if l%2 == 0:
                col = "#eeeeee"
            else:
                col = "#ffffff"
            iter = self.model.append()
            self.model.set(iter, MOD_COLUMN_SONG,
                              unicode(self.plist[l]['song'], 'Latin-1'),
                              MOD_COLUMN_REF, l,
                              MOD_COLUMN_BG, col)
        self.update_current()

    def update_current(self):
        i = iter = self.model.get_iter_root ()
        if self.plist.selected!=None:
            while 1:
                if (self.plist.selected ==
                    self.model.get_value(i, MOD_COLUMN_REF)):
                    # iter is now set to the column that's playing
                    path = self.model.get_path(i)
                    #self.treeview.scroll_to_cell (path, self.column, 1,
                    #                                 0.5, 0)

                    self.treeview.set_cursor (path, self.column, 0)
                    self.treeview.grab_focus()

                    selection = self.treeview.get_selection()
                    selection.select_iter(i)
                    return

                i = self.model.iter_next (i)
                if not i: break






class PlaylistWindow:

    def __init__(self, playlist):
        assert(os.path.isfile(GLADE_FILE))

        self.xml = libglade.XML(GLADE_FILE, root="playlist")

        self.plist = playlist

        # convenience
        self.window = self.xml.get_widget("playlist")
        self.status = self.xml.get_widget("playlist_status_top")
        self.prompt = self.xml.get_widget("playlist_prompt")
        self.entry = self.xml.get_widget("playlist_entry")
        self.scroll = self.xml.get_widget("scrolledwindow")
        self.song_progress_label = self.xml.get_widget("song_progress_label")
        self.song_progress_slider = self.xml.get_widget("song_progress_slider")
        self.song_progress_adj = self.song_progress_slider.get_adjustment()

        ui.add_callback('song', self.set_song)
        ui.add_callback('status', self.set_status)
        ui.add_callback('notice', self.notice)
        ui.add_callback('time', self.set_time)

        self.set_song()
        self.set_status()

        # add gtk to the event loop
        multiplex.add_idle(self._main_iter)

        # interesting hack to get a list of methods of this class
        # (class.__dict__ gives a list of funtions not bound to an instance)

        dict = {}
        for key in self.__class__.__dict__.keys():
            if key.find("on_") != 0: continue

            val = getattr(self, key)
            if isinstance(val, types.MethodType):
                dict.update({key: val})

        for (k, v) in dict.items():
            self.xml.signal_connect(k, v)

        # connect the adjustment value_changed signal
#         self.song_progress_adj.connect("value_changed", \
#                                        self.on_progress_adj_val_changed)

        # install our playlist list widget
        self.listview = PlaylistList(playlist)
        self.scroll.add(self.listview.get_widget())

        self.plist.add_hook('playing', self.set_song)

        self.old_current = 0

        self.window.show_all()

    def playlist_refresh(self):
        """ called when we refresh the playlist """

        self.listview.draw_playlist(self.list)
        self.set_song()

    def on_playlist_entry_activate(self, *args):
        """ when the user hits enter

        our response here depends on the mode """

        text = self.entry.get_text()

        path = self.treeview.get_row_data(self.selected_row).path
        muse.player.play(self.plist.jump_to_field(-1, 'path', path))

    def on_refresh_activate(self, *args):
        self.refresh()

    def on_jump_to_playing_activate(self, *args):
        pass
#self.treeview.moveto(self.playing_row, 0, 0.5, 0)

    def on_jump_to_selected_activate(self, *args):
        self.treeview.moveto(self.selected_row, 0, 0.5, 0)

    def on_pause_activate(self, *args):
        muse.player.pause()

    def on_selected_to_playing_activate(self, *args):
        self.treeview.select_row(self.playing_row, 0)
        self.on_jump_to_selected_activate()

    def on_select_first_activate(self, *args):
        self.treeview.select_row(0, 0)
        self.on_jump_to_selected_activate()

    def on_select_last_activate(self, *args):
        self.treeview.select_row(self.last_row, 0)
        self.on_jump_to_selected_activate()

    def on_play_selected_activate(self, *args):
        muse.player.play(self.plist.selected)

    def on_cue_forward_activate(self, *args):
        muse.player.time_seek(muse.player.time_tell() + 10)

    def on_cue_backward_activate(self, *args):
        muse.player.time_seek(muse.player.time_tell() - 10)

    def on_interactive_search_activate(self, *args):
        IsearchWindow()

    def on_song_progress_slider_button_release_event(self, *args):

        val = self.song_progress_adj.value

        # don't attempt to do anything if we set the value
        try:
            if val == self.old_value:
                return
        except AttributeError:
            pass

        muse.player.time_seek(muse.player.time_total() * val)

    def _main_iter(self):
        while gtk.events_pending():
            gtk.main_iteration_do(0)
        return 1

    def notice(self, msg):
        """ shows a low-priority message - the sort of thing to appear briefly
        in a status bar, for instance."""
        pass

        id = self.status.get_context_id(msg)
        self.status.push(id, msg)

##        # now we add a timer to clear the notice in ~ 1.5 seconds
        multiplex.set_timer("gtk_notice_timer" + repr(time.time()), 2, \
                            lambda x=self, id=id: x.status.pop(id))

    def set_song(self):
        """ called when a new is playing """
        if self.plist.playing:
            self.entry.set_text(
                unicode(self.plist.playing['song'], "Latin-1"))

        try:
##            # clear the hilight on the old song
            #self.treeview.set_background(self.playing_row,
            #self.white)
            pass
        except AttributeError:
            # no current playing row
            pass

        return
        # find new playing row
        self.playing_row = self.treeview.find_row_from_data( \
            self.plist.pl)

        # hilight new row, and jump to it if it's not visible
        if self.playing_row != -1:
            self.treeview.set_background(self.playing_row, self.hilight)
            vis = self.treeview.row_is_visible(self.playing_row)

            if vis != GTK.VISIBILITY_FULL:
                self.treeview.moveto(self.playing_row, 0, 0.5, 0)

        # move the selected row to the currently playing row
        self.treeview.select_row(self.playing_row, 0)

    def set_status(self):
        try:
            text = muse.player.status
        except:
            text = "stopped"
        self.prompt.set_text(unicode(text, "Latin-1"))

    def set_time(self):
        """given a time in seconds, update the display"""

        secs = muse.player.time_tell()
        total = muse.player.time_total()

        position = secs/total

        if secs != 0:
            str = "%.2d:%.2d" % (int(secs/60),secs%60)
        else:
            str = "     "

        self.song_progress_label.set_text(str)

        self.song_progress_adj.set_value(position)
        self.song_progress_adj.emit("value_changed")

        self.set_position = position

class new(ui.plugin):
    """ provides an abstract class for a interface plugin, for muse"""

    def _onexit(self):
        """ called by atexit, clean up the screen and prevent further
        modifications """
        pass

    def __init__(self):

        ui.plugin.__init__(self)

        # plugin requisite stuff
        self.plugin_type = "ui"
        self.plugin_desc = "gtk interface"
        self.plugin_name = "gtk"

        self.playlist = PlaylistWindow(muse.plist)
        #queue = PlaylistWindow(muse.pqueue)

    def deinit (self):
        self.playlist.window.destroy()
