#!/usr/bin/env python
#
# $Id: ui_osd.py,v 1.1.1.1 2003/02/11 06:07:41 resolve Exp $
#
# Started 11/12/01 by Damien Elmes <resolve@repose.cx>
# This file is licensed under the GPL.
#

"""
Plugin for muse which delivers song information on a Matrix Orbital LCD screen,
and keypad watching, for remote commands.
"""

import os
import types

import muse
import ui
import multiplex
import config
import mp3meta
import control

from debug import debug

try:
    import pyosd
except ImportError:
    debug("pyosd library not installed, ui_osd offline.")
    pyosd=0

PLUGIN_TYPE="ui"

if pyosd:
    config_defaults = {
        "font":    pyosd.default_font,
        "status_colour":  "#FFFFFF",
        "song_colour":  "#FFBBBB",
        }

    class new(ui.plugin):
        def __init__(self):
            ui.plugin.__init__(self)

            self.plugin_type = "ui"
            self.plugin_desc = "on screen display"
            self.plugin_name = "osd"

            config.set_defaults("ui_osd", config_defaults)

            self.font = config.get("ui_osd", "font")
            self.song_colour = config.get("ui_osd", "song_colour")
            self.status_colour = config.get("ui_osd", "status_colour")

            self.osd_is_on = 0

            try:

                self.osd = pyosd.osd(self.font, self.song_colour,
                                     pos=pyosd.POS_BOT,
                                     shadow=0, timeout=5, offset=60)
                self.status = pyosd.osd(
                    "-adobe-helvetica-medium-r-normal-*-*-240-*-*-p-*-iso8859-1",
                    self.status_colour,
                    pos=pyosd.POS_BOT,
                    shadow=0, timeout=5, offset=50)

                self.album = pyosd.osd("-adobe-helvetica-medium-r-normal-*-*-240-*-*-p-*-iso8859-1", "#CCCCFF",
                                       pos=pyosd.POS_BOT, shadow=0, timeout=5,
                                       offset=-20)

            except pyosd.error, e:
                debug("pyosd not loaded: " + str(e))
                return

            muse.plist.add_hook('playing', self.set_song)
            #ui.add_callback('status', self.set_status)
            ui.add_callback('time', self.set_time)
            ui.add_callback('notice', self.notice)

            control.control_list["osd"] = self.osd_control

        def osd_control(self, args):

            if args:
                arg = args[0]
            else:
                return

            if arg == "on":
                self.osd_on()
            elif arg == "off":
                self.osd_off()
            elif arg == "toggle":
                self.osd_toggle()
            else:
                print "warning, unhandled command in osd_control: " + arg

        def osd_on(self):

            self.osd_is_on = 1

            self.osd.set_timeout(-1)
            self.album.set_timeout(-1)

            self.set_song()


        def osd_off(self):

            self.osd_is_on = 0

            self.osd.set_timeout(3)
            self.album.set_timeout(3)

        def osd_toggle(self):

            if self.osd_is_on:
                self.osd_off()
            else:
                self.osd_on()

        def notice(self, msg):
            """ shows a low-priority message - the sort of thing to appear
            briefly in a status bar, for instance."""
            pass

        def set_time(self):
            pass
            #self.scroll.display(100* (muse.player.time_tell() /
            #                    muse.player.time_total()), pyosd.TYPE_PERCENT)

        def set_status(self):
            self.status.display(muse.player.status)
            pass

        def set_song(self):
            if not len(muse.plist): return

            if muse.plist.playing:
                item = muse.plist.playing

                self.osd.display("")
                self.osd.display(os.path.basename(item['path']), pyosd.TYPE_STRING, 1)

                string = ""

                if item.get('track'):
                    string += "track %d" % item['track']

                if item.get('playlist'):
                    if string:
                        string += " "

                    pl = mp3meta.nice_case(
                        os.path.basename(item['playlist']).replace(".m3u",
                        ""))

                    try:
                        (artist, title) = pl.split(" - ")
                    except ValueError:
                        # couldn't split
                        artist = pl
                        title = ""

                    if artist == item['artist']:
                        # omit album if it's the same
                        string += "from the album " + title
                    else:
                        string += "from the album " + pl

                if string:
                    self.album.display("(" + string + ")")

else:
    class new:
        pass














