#!/usr/bin/env python
#
# $Id: ui_remote_unix.py,v 1.1.1.1 2003/02/11 06:07:41 resolve Exp $
#
# control muse via writing/reading to a unix socket.
#
# Started 5/8/01 by Damien Elmes <resolve@repose.cx>
# This file is licensed under the GPL.
#

import os
import socket
import string

import multiplex
import ui
import config
import control

from debug import debug

PLUGIN_TYPE="ui"

config_defaults = {
    "port": 8008
    }

config.set_defaults("ui_tcp", config_defaults)

class new(ui.plugin):
    """ provides an abstract class for a interface plugin, for muse"""

    def __init__(self):
        ui.plugin.__init__(self)

        self.plugin_type = "ui"
        self.plugin_desc = "control via a tcp port"
        self.plugin_name = "tcp"

        self.collected = ""

        self.port = int(config.get("ui_tcp", "port"))
        self.conns = {}

        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.setblocking(0)
        self.sock.setsockopt (socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.sock.bind(("", self.port))
        self.sock.listen(1)

        # good, we're non blocking now. set up a hook for accept calls

        multiplex.add_watch(self.sock.fileno(), multiplex.POLLIN,
                            self.on_accept)

    def __del__(self):
        pass

    def on_accept(self, fd):
        try:
            x = self.sock.accept()
        except None:
            # shouldn't happen
            return

        sock = x[0]

        self.conns[sock.fileno()] = sock

        multiplex.add_watch(sock.fileno(), multiplex.POLLIN |
                            multiplex.POLLHUP, self.on_recv)

    def on_recv(self, fd):
        """ Called when there is action on a socket """

        try:
            res = self.conns[fd].recv(256)
        except socket.error:
            res = None

        if not res:
            # no more data to read, clean up this socket.
            multiplex.del_watch(fd)
            self.conns[fd].close()
            del self.conns[fd]
            return

        if res.find("\n")==-1:
            self.collected = res
            return
        else:
            if self.collected:
                res = self.collected + res
                self.collected = ""

        print "ui_tcp: recv=%s" % string.strip(res)
        out = str(control.cmd(res))
        try:
            if out != None:
                self.conns[fd].sendall(out.strip() + "\n")
        except socket.error:
            return


    def notice(self, msg): pass


if __name__ == "__main__":

    f = new()














